// ------------------------------------------------------------
// OO Wrapper For Boutell.com gd library
//
// Mauricio De Simone
// mdesimon@uwaterloo.ca
// ------------------------------------------------------------

#ifndef gdimage_pass
#define gdimage_pass

#include <fstream.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

extern "C" {
#  include "gd.h"
}

#include "dfe.H"

class gImage;

class Color
{
public:
  Color();

  Color(int me);
  // Placeholder
  Color(int red, int green, int blue);


  void inImage(gImage*);
  // Behave correctly with original library


  operator int();

private:
  int _me;
  gImage* _image;
  int _red;
  int _green;
  int _blue;
};




//
// Image class
//

class gImage
{
public:
  // Posible file types
  enum Type {
    GIF,
    GD,
    XBM
  };

  static int maxColors;

  // Constructors/Destructor
  gImage(int owner = 0);
  // Construct from dimensions
  gImage(int sx, int sy, Color& background = Color(0,0,0), int owner = FALSE);
  // Construct from file
  gImage(const char* filename, Type type = GIF, int owner = FALSE);
  // Reclaim memory
  gImage(const gImage& theOther);
  ~gImage();


  // I/O
  void write(const char * filename);


  int dx() const;
  int dy() const;


  // Pixel Manipulation
  void setPixel(int x, int y, Color& color);
  Color getPixel(int x, int y);
  int getPixelIndex(int x, int y);


  // Lines 
  void line(int x1, int y1, int x2, int y2, Color& color);


  // Rectangles
  enum RecType {
    NORMAL,
    FILLED
  };
  void rectangle(int x1, int y1, int x2, int y2, Color& color, RecType type = NORMAL);



  //
  // Fill funcs
  //
  void fill(int x, int y, Color& color);


  friend class Color;


  // protected:
  gdImage* _image;

private:
  int _owner;

#ifdef CC_TOOLS
  Mutex _me;
#endif
};


//
// Grey Scale Image
//
class GreyScalegImage : 
  public gImage
{
public:
  GreyScalegImage(const gImage& rhs);

protected:
  void initializePalette();
};



//
// Masks
//

class Mask
{
public:
  Mask(int dx, int dy, int value = 0);
  ~Mask();

  void set(int x, int y, int value);
  void set(int* mask, int dx, int dy);
  int get(int x, int y);

  int dx() const;
  int dy() const;

private:
  int* _mask;
  int _dx;
  int _dy;
};

#endif



