#include <cstdlib>
#include <ctime>
#include <fstream>
#include <iostream>
#include <sys/time.h>

#include "Image.h"

using namespace std;

double getClock()
{
    struct timeval tp;
    struct timezone tzp;

    gettimeofday(&tp, &tzp);

    return (double)tp.tv_sec+((double)tp.tv_usec/1000000.0);
}

// The following is the algorithm for convoluting the given
// image through the application of the mask.

void Convolute (Image& imageIn, Image& mask, Image& imageOut)
{
        int idx, jdx;
        int ms,im,val;
        unsigned char* tmp;

        for (int i = 0; i < imageIn.dx(); i++){
                for (int j = 0; j < imageIn.dy(); j++){
                        val = im = 0;
                        for (int m = 0; m < mask.dx(); m++){
                                for (int n = 0; n < mask.dy(); n++){
                                        ms = (signed char) mask.get(m,n);
                                        idx = i - m;
                                        jdx = j - n;
                                        if (idx >= 0 && jdx >= 0)
                                                im = imageIn.get(idx,jdx);
                                        val += ms * im;
                                }
                        }
                        if (val > 255) val = 255;
                        if (val < 0) val = 0;

                        imageOut.set(i,j,val);
                }
        }

};

main()
{
	double start, end;
        FILE *I_in, *I_out;
        gdImagePtr gdIn = 0;
        gdImagePtr gdOut = 0;
	int y;

        I_in = fopen ("Input.gif", "rb");
        if (!I_in) {
              cout << "Error: Cannot open input file." << endl;
              exit(-1);
        }
        gdIn = gdImageCreateFromGif(I_in);
        fclose(I_in);
        if (!gdIn) {
              cout << "Error: Not a valid input gif file." << endl;
              exit(-1);
        }

        Image imageIn(gdIn);
        Image imageOut(imageIn);

	ifstream M_in("Mask.in", ios::in);
        int mx, my;
        M_in >> mx >> my;
        Image Mask(mx, my);
        int tmp;
        for (y = 0; y < my; y++)
               for (int x = 0; x < mx; x++){
                       M_in >> tmp;
                       Mask.set(x,y,tmp);
               }
        // Start actual computation here.....
   	// Measure the time for sequential computation.

        start = getClock();
        Convolute(imageIn, Mask, imageOut);
        end = getClock();

	cerr << "Total Time: " << (end-start) << endl;

        gdOut = gdImageCreate(imageOut.dx(), imageOut.dy());
        if (!gdOut){
              cout << "Error: Unable to create output gd file!" << endl;
              exit(-1);
        }
        for (int i = 0; i < gdMaxColors; i++) {
              gdImageColorAllocate(gdOut, gdImageRed(gdIn, i), gdImageGreen(gdIn, i),gdImageBlue(gdIn, i));
        }
        for (y = 0; y < imageOut.dy(); y++)
              for (int x = 0; x < imageOut.dx(); x++)
                     gdImageSetPixel(gdOut, x, y, imageOut.get(x,y));

        I_out = fopen ("Output.gif", "wb");
        if (!I_out){
              cout << "Error: Unable to create output gif file." << endl;
              exit (-1);
        }
        gdImageGif(gdOut, I_out);
        fclose(I_out);
        gdImageDestroy(gdIn);
        gdImageDestroy(gdOut);
}

